@@ -22,6 +22,67 @@
 #' @import stats
 #' @import utils
 
+
+## function TnT()
+# this is the most basic preparation of a text for use with lex.div()
+TnT <- function(txt, force.lang=NULL, corp.rm.class="nonpunct",
+    corp.rm.tag=c(), case.sens=FALSE, lemmatize=FALSE, keep.tokens=FALSE, quiet=FALSE){
+  result <- list()
+  # get class kRp.tagged from txt object
+  # the internal function tag.kRp.txt() will return the object unchanged if it
+  # is already tagged, so it's safe to call it with the lang set here
+  tagged.text <- tag.kRp.txt(
+    txt,
+    lang=force.lang,
+    objects.only=FALSE,
+    # 'format' will only be used by tokenize() or treetag()
+    format="obj"
+  )
+  # set the language definition
+  lang <- language.setting(tagged.text, force.lang)
+  if(!isTRUE(quiet)){
+    cat("Language: \"",lang,"\"\n", sep="")
+  } else {}
+
+  if(identical(corp.rm.class, "nonpunct")){
+    corp.rm.class <- kRp.POS.tags(lang, tags=c("punct","sentc"), list.classes=TRUE)
+  } else {}
+
+  # calling internal function tagged.txt.rm.classes()
+  txt.all.clean <- tagged.txt.rm.classes(taggedText(tagged.text),
+      # "lemma" only affects the results if "as.vector=TRUE"
+      # so lemmatizing will be done by type.freq() below
+      lemma=FALSE, lang=lang,
+      corp.rm.class=corp.rm.class,
+      corp.rm.tag=corp.rm.tag,
+      as.vector=FALSE)
+  if(isTRUE(lemmatize)){
+    result[["txt.all.tokens"]] <- txt.all.clean[,"lemma"]
+  } else {
+    result[["txt.all.tokens"]] <- txt.all.clean[,"token"]
+  }
+  if(isTRUE(keep.tokens)){
+    result[["txt.type.freq"]] <- type.freq(txt.all.clean, case.sens=case.sens, lemma=FALSE)
+    result[["txt.lemma.freq"]] <- type.freq(txt.all.clean, case.sens=case.sens, lemma=TRUE, fail.if.no.lemmas=lemmatize)
+    result[["txt.all.types"]] <- result[["txt.type.freq"]][,"type"]
+    result[["txt.all.lemmas"]] <- result[["txt.lemma.freq"]][,"type"]
+  } else {
+    result[["txt.type.freq"]] <- result[["txt.lemma.freq"]] <- NULL
+    result[["txt.all.types"]] <- type.freq(txt.all.clean, case.sens=case.sens, lemma=FALSE, vector.only=TRUE)
+    result[["txt.all.lemmas"]] <- type.freq(txt.all.clean, case.sens=case.sens, lemma=TRUE, fail.if.no.lemmas=lemmatize, vector.only=TRUE)
+  }
+  if(!isTRUE(case.sens)){
+    result[["txt.all.tokens"]] <- tolower(result[["txt.all.tokens"]])
+  } else {}
+  result[["num.all.tokens"]] <- length(result[["txt.all.tokens"]])
+  result[["num.all.types"]] <- length(result[["txt.all.types"]])
+  result[["num.all.lemmas"]] <- length(result[["txt.all.lemmas"]])
+  result[["num.type.or.lemma"]] <- ifelse(isTRUE(lemmatize), result[["num.all.lemmas"]], result[["num.all.types"]])
+
+  return(result)
+} ## end function TnT()
+
+
 # this internal function does the real lexical diversity calculations,
 # so it's mostly called by lex.div()
 
@@ -29,7 +90,7 @@
 ## if this signature changes, check lex.div() as well! ##
 #########################################################
 
-kRp.lex.div.formulae <- function(txt, segment=100, factor.size=0.72, min.tokens=9, rand.sample=42, window=100,
+kRp.lex.div.formulae <- function(txt, segment=100, factor.size=0.72, min.tokens=9, MTLDMA.steps=1, rand.sample=42, window=100,
     case.sens=FALSE, lemmatize=FALSE, detailed=FALSE,
     measure=c("TTR","MSTTR","MATTR","C","R","CTTR","U","S","K","Maas","HD-D","MTLD","MTLD-MA"),
     char=c("TTR","MATTR","C","R","CTTR","U","S","K","Maas","HD-D","MTLD","MTLD-MA"),
@@ -86,44 +147,69 @@ kRp.lex.div.formulae <- function(txt, segment=100, factor.size=0.72, min.tokens=
       stop(simpleError(paste("You didn't specify at least one valid measure or characteristic!")))
   } else {}
 
-  # get class kRp.tagged from txt object
-  # the internal function tag.kRp.txt() will return the object unchanged if it
-  # is already tagged, so it's safe to call it with the lang set here
-  tagged.text <- tag.kRp.txt(txt, lang=force.lang, objects.only=TRUE)
-  # set the language definition
-  lang <- language.setting(tagged.text, force.lang)
-  if(!isTRUE(quiet)){
-    cat("Language: \"",lang,"\"\n", sep="")
-  } else {}
+  basicTnT <- TnT(
+    txt=txt,
+    force.lang=force.lang,
+    corp.rm.class=corp.rm.class,
+    corp.rm.tag=corp.rm.tag,
+    case.sens=case.sens,
+    lemmatize=lemmatize,
+    keep.tokens=keep.tokens,
+    quiet=quiet
+  )
+  txt.all.tokens <- basicTnT[["txt.all.tokens"]]
+  txt.all.types <- basicTnT[["txt.all.types"]]
+  txt.all.lemmas <- basicTnT[["txt.all.lemmas"]]
+  txt.type.freq <- basicTnT[["txt.type.freq"]]
+  txt.lemma.freq <- basicTnT[["txt.lemma.freq"]]
+  num.all.tokens <- basicTnT[["num.all.tokens"]]
+  num.all.types <- basicTnT[["num.all.types"]]
+  num.all.lemmas <- basicTnT[["num.all.lemmas"]]
+  num.type.or.lemma <- basicTnT[["num.type.or.lemma"]]
 
-  if(identical(corp.rm.class, "nonpunct")){
-    corp.rm.class <- kRp.POS.tags(lang, tags=c("punct","sentc"), list.classes=TRUE)
-  } else {}
-
-  # calling internal function tagged.txt.rm.classes()
-  txt.all.clean <- tagged.txt.rm.classes(tagged.text@TT.res,
-      # "lemma" only affects the results if "as.vector=TRUE"
-      # so lemmatizing will be done by type.freq() below
-      lemma=FALSE, lang=lang,
-      corp.rm.class=corp.rm.class,
-      corp.rm.tag=corp.rm.tag,
-      as.vector=FALSE)
-  if(isTRUE(lemmatize)){
-    txt.all.tokens <- txt.all.clean[,"lemma"]
-  } else {
-    txt.all.tokens <- txt.all.clean[,"token"]
-  }
-  txt.type.freq <- type.freq(txt.all.clean, case.sens=case.sens, lemma=FALSE)
-  txt.lemma.freq <- type.freq(txt.all.clean, case.sens=case.sens, lemma=TRUE, fail.if.no.lemmas=lemmatize)
-  txt.all.types <- txt.type.freq[,"type"]
-  txt.all.lemmas <- txt.lemma.freq[,"type"]
-  if(!isTRUE(case.sens)){
-    txt.all.tokens <- tolower(txt.all.tokens)
-  } else {}
-  num.all.tokens <- length(txt.all.tokens)
-  num.all.types <- length(txt.all.types)
-  num.all.lemmas <- length(txt.all.lemmas)
-  num.type.or.lemma <- ifelse(isTRUE(lemmatize), num.all.lemmas, num.all.types)
+#   # get class kRp.tagged from txt object
+#   # the internal function tag.kRp.txt() will return the object unchanged if it
+#   # is already tagged, so it's safe to call it with the lang set here
+#   tagged.text <- tag.kRp.txt(txt, tagger="tokenize", lang=force.lang, objects.only=FALSE)
+#   # set the language definition
+#   lang <- language.setting(tagged.text, force.lang)
+#   if(!isTRUE(quiet)){
+#     cat("Language: \"",lang,"\"\n", sep="")
+#   } else {}
+# 
+#   if(identical(corp.rm.class, "nonpunct")){
+#     corp.rm.class <- kRp.POS.tags(lang, tags=c("punct","sentc"), list.classes=TRUE)
+#   } else {}
+# 
+#   # calling internal function tagged.txt.rm.classes()
+#   txt.all.clean <- tagged.txt.rm.classes(tagged.text@TT.res,
+#       # "lemma" only affects the results if "as.vector=TRUE"
+#       # so lemmatizing will be done by type.freq() below
+#       lemma=FALSE, lang=lang,
+#       corp.rm.class=corp.rm.class,
+#       corp.rm.tag=corp.rm.tag,
+#       as.vector=FALSE)
+#   if(isTRUE(lemmatize)){
+#     txt.all.tokens <- txt.all.clean[,"lemma"]
+#   } else {
+#     txt.all.tokens <- txt.all.clean[,"token"]
+#   }
+#   if(isTRUE(keep.tokens)){
+#     txt.type.freq <- type.freq(txt.all.clean, case.sens=case.sens, lemma=FALSE)
+#     txt.lemma.freq <- type.freq(txt.all.clean, case.sens=case.sens, lemma=TRUE, fail.if.no.lemmas=lemmatize)
+#     txt.all.types <- txt.type.freq[,"type"]
+#     txt.all.lemmas <- txt.lemma.freq[,"type"]
+#   } else {
+#     txt.all.types <- type.freq(txt.all.clean, case.sens=case.sens, lemma=FALSE, vector.only=TRUE)
+#     txt.all.lemmas <- type.freq(txt.all.clean, case.sens=case.sens, lemma=TRUE, fail.if.no.lemmas=lemmatize, vector.only=TRUE)
+#   }
+#   if(!isTRUE(case.sens)){
+#     txt.all.tokens <- tolower(txt.all.tokens)
+#   } else {}
+#   num.all.tokens <- length(txt.all.tokens)
+#   num.all.types <- length(txt.all.types)
+#   num.all.lemmas <- length(txt.all.lemmas)
+#   num.type.or.lemma <- ifelse(isTRUE(lemmatize), num.all.lemmas, num.all.types)
   # some sanity checks
   if(num.all.tokens < 100){
     warning("Text is relatively short (<100 tokens), results are probably not reliable!", call.=FALSE)
@@ -203,12 +289,12 @@ kRp.lex.div.formulae <- function(txt, segment=100, factor.size=0.72, min.tokens=
 
   ## Yule's K, frequency correction
   if("K" %in% measure){
-    lex.div.results@K.ld <- k.calc(txt.all.tokens)
+    lex.div.results@K.ld <- k.calc(txt.all.tokens, txt.types=txt.all.types)
   } else {}
 
   ## calculate HD-D
   if("HD-D" %in% measure){
-    lex.div.results@HDD <- hdd.calc(txt.all.tokens, drawn=rand.sample)
+    lex.div.results@HDD <- hdd.calc(txt.all.tokens, drawn=rand.sample, hdd.types=txt.all.types)
   } else {}
 
   ## calculate MTLD
@@ -221,63 +307,38 @@ kRp.lex.div.formulae <- function(txt, segment=100, factor.size=0.72, min.tokens=
     # characteristics need detailed results, so discard another setting if present to speed up things
     # the alternative would be to calculate this twice, so it's a no-brainer
     # to comply with user preferences, we'll drop the detailed stuff again, see MTLDMA.char section!
-    if("MTLD-MA" %in% char && !isTRUE(detailed)){
+    if("MTLD-MA" %in% char){
       detailed.mtldma <- TRUE
     } else {
       detailed.mtldma <- detailed
     }
-    lex.div.results@MTLDMA <- MTLDMA.calc(txt.all.tokens, factor.size=factor.size, num.tokens=num.all.tokens, min.tokens=min.tokens,
+    lex.div.results@MTLDMA <- MTLDMA.calc(txt.all.tokens, factor.size=factor.size, num.tokens=num.all.tokens, min.tokens=min.tokens, steps=MTLDMA.steps,
       detailed=detailed.mtldma, quiet=quiet)
   } else {}
 
   ## calculate TTR, C, R, CTTR, U, S and Maas characteristics
-  # set up the base function
-  ttr.calc.chars <- function(txt.tokens, type="TTR", log.base=log.base){
-    if(!isTRUE(quiet)){
-      message(paste0(type, ".char: Calculate ",type," values"))
-      # give some feedback, so we know the machine didn't just freeze...
-      prgBar <- txtProgressBar(min=0, max=num.all.steps, style=3)
-    } else {}
-    char.results <- t(sapply(1:num.all.steps, function(x){
-          curr.token <- x * char.steps
-          if(!isTRUE(quiet)){
-            # update progress bar
-            setTxtProgressBar(prgBar, x)
-          } else {}
-          char.temp <- c(token=curr.token, value=ttr.calc(txt.tokens=txt.all.tokens[1:curr.token], type=type, log.base=log.base))
-          return(char.temp)
-        }
-      ))
-    if(!isTRUE(quiet)){
-      # close prograss bar
-      close(prgBar)
-    } else {}
-    return(char.results)
-  }
-
-  # do the actual calculations
   if("TTR" %in% char){
-    lex.div.results@TTR.char <- ttr.calc.chars(txt.all.tokens, type="TTR")
+    lex.div.results@TTR.char <- ttr.calc.chars(txt.all.tokens, type="TTR", steps=num.all.steps, char.steps=char.steps, quiet=quiet)
   } else {}
 
   if("C" %in% char){
-    lex.div.results@C.char <- ttr.calc.chars(txt.all.tokens, type="C", log.base=log.base)
+    lex.div.results@C.char <- ttr.calc.chars(txt.all.tokens, type="C", log.base=log.base, steps=num.all.steps, char.steps=char.steps, quiet=quiet)
   } else {}
 
   if("R" %in% char){
-    lex.div.results@R.char <- ttr.calc.chars(txt.all.tokens, type="R")
+    lex.div.results@R.char <- ttr.calc.chars(txt.all.tokens, type="R", steps=num.all.steps, char.steps=char.steps, quiet=quiet)
   } else {}
 
   if("CTTR" %in% char){
-    lex.div.results@CTTR.char <- ttr.calc.chars(txt.all.tokens, type="CTTR")
+    lex.div.results@CTTR.char <- ttr.calc.chars(txt.all.tokens, type="CTTR", steps=num.all.steps, char.steps=char.steps, quiet=quiet)
   } else {}
 
   if("U" %in% char){
-    lex.div.results@U.char <- ttr.calc.chars(txt.all.tokens, type="U", log.base=log.base)
+    lex.div.results@U.char <- ttr.calc.chars(txt.all.tokens, type="U", log.base=log.base, steps=num.all.steps, char.steps=char.steps, quiet=quiet)
   } else {}
 
   if("S" %in% char){
-    lex.div.results@S.char <- ttr.calc.chars(txt.all.tokens, type="S", log.base=log.base)
+    lex.div.results@S.char <- ttr.calc.chars(txt.all.tokens, type="S", log.base=log.base, steps=num.all.steps, char.steps=char.steps, quiet=quiet)
   } else {}
 
   if("MATTR" %in% char && num.all.tokens > window){
@@ -294,7 +355,7 @@ kRp.lex.div.formulae <- function(txt, segment=100, factor.size=0.72, min.tokens=
   } else {}
 
   if("Maas" %in% char){
-    lex.div.results@Maas.char <- ttr.calc.chars(txt.all.tokens, type="Maas", log.base=log.base)
+    lex.div.results@Maas.char <- ttr.calc.chars(txt.all.tokens, type="Maas", log.base=log.base, steps=num.all.steps, char.steps=char.steps, quiet=quiet)
     maas.lgV.chars <- function(base){
       if(!isTRUE(quiet)){
         # give some feedback, so we know the machine didn't just freeze...
@@ -475,7 +536,7 @@ kRp.lex.div.formulae <- function(txt, segment=100, factor.size=0.72, min.tokens=
     "\n  Use the results with caution, even if they seem plausible!"), call.=FALSE)
   } else {}
   return(lex.div.results)
-}
+} ## end function kRp.lex.div.formulae()
 
 
 ## function ttr.calc()
@@ -523,16 +584,19 @@ ttr.calc <- function(txt.tokens=NULL, txt.types=NULL, num.tokens=NULL, num.types
 
 ## function k.calc()
 # function to calculate Yule's K
-k.calc <- function(txt.tokens){
+k.calc <- function(txt.tokens, txt.types=NULL){
   N <- length(txt.tokens)
-  txt.types <- unique(txt.tokens)
+  if(is.null(txt.types)){
+    stopifnot(!is.null(txt.tokens))
+    txt.types <- unique(txt.tokens)
+  } else {}
   # first analize types for their frequencies
   type.freqs <- sapply(txt.types, function(x){
-      return(sum(match(txt.tokens, x), na.rm=TRUE))
+      return(sum(txt.tokens %in% x, na.rm=TRUE))
     })
   # now count the frequencies of frequencies
   freq.freqs <- t(sapply(unique(type.freqs), function(x){
-      frq.frq <- sum(match(type.freqs, x), na.rm=TRUE)
+      frq.frq <- sum(type.freqs %in% x, na.rm=TRUE)
       return(c(X=x, fx=frq.frq))
     }))
   # calculate the sum of all fx * X^2
@@ -547,13 +611,16 @@ k.calc <- function(txt.tokens){
 
 ## function hdd.calc()
 # function to calculate HD-D
-hdd.calc <- function(hdd.tokens, drawn){
-  hdd.types <- unique(hdd.tokens)
+hdd.calc <- function(hdd.tokens, drawn, hdd.types=NULL){
+  if(is.null(hdd.types)){
+    stopifnot(!is.null(hdd.tokens))
+    hdd.types <- unique(hdd.tokens)
+  } else {}
   num.hdd.tokens <- length(hdd.tokens)
   num.hdd.types <- length(hdd.types)
   # get probability from hypergeometric distribution for each type
   hdd.type.probs <- sapply(hdd.types, function(x){
-      num.types.in.tokens <- sum(match(hdd.tokens, x), na.rm=TRUE)
+      num.types.in.tokens <- sum(hdd.tokens %in% x, na.rm=TRUE)
       num.nontypes <- num.hdd.tokens - num.types.in.tokens
       # probability of having the type at least once is the inverse of not drawing it at all
       # if 'drawn' is larger than number of tokens, set probability to 1
@@ -826,7 +893,7 @@ mtld.sub.calc <- function(txt.tokens, factor.size, num.tokens=NULL, detailed=FAL
 
 ## function MTLD.calc()
 # calculate MTLD
-# to get only the backwards data (used for characteristics), set "back.only=TRUE)"
+# to get only the backwards data (used for characteristics), set "back.only=TRUE"
 MTLD.calc <- function(txt.tokens, factor.size, num.tokens=NULL, back.only=FALSE, detailed=FALSE){
   if(is.null(num.tokens)){
     num.tokens <- length(txt.tokens)
@@ -866,7 +933,7 @@ MTLD.calc <- function(txt.tokens, factor.size, num.tokens=NULL, back.only=FALSE,
 
 
 ## function MTLDMA.calc()
-MTLDMA.calc <- function(txt.tokens, factor.size, num.tokens=NULL, min.tokens=9, detailed=FALSE, quiet=TRUE){
+MTLDMA.calc <- function(txt.tokens, factor.size, num.tokens=NULL, min.tokens=9, steps=1, detailed=FALSE, quiet=TRUE){
   if(is.null(num.tokens)){
     num.tokens <- length(txt.tokens)
   } else {}
@@ -879,15 +946,13 @@ MTLDMA.calc <- function(txt.tokens, factor.size, num.tokens=NULL, min.tokens=9,
 
   # we'll simply run mtld.sub.calc() in a loop with changing start tokens
   # and then average the factor results
-  nextToken <- 1
+  start.with <- 1
   mtldma.list <- list()
-  while (nextToken < num.tokens){
+  while (start.with < num.tokens){
     if(!isTRUE(quiet)){
       # update progress bar
-      setTxtProgressBar(prgBar, nextToken)
+      setTxtProgressBar(prgBar, start.with)
     } else {}
-    start.with <- nextToken
-    num.tokens.sub <- num.tokens - start.with + 1
     mtldma.fct.tokens <- txt.tokens[start.with:num.tokens]
     sub.calc.result <- mtld.sub.nodata(txt.tokens=mtldma.fct.tokens, factor.size=factor.size,
       stopAtFirstFactor=TRUE, detailed=detailed, offset=(start.with - 1))
@@ -896,7 +961,7 @@ MTLDMA.calc <- function(txt.tokens, factor.size, num.tokens=NULL, min.tokens=9,
       break
     } else {}
     mtldma.list[[start.with]] <- sub.calc.result
-    nextToken <- nextToken + 1
+    start.with <- start.with + steps
   }
   if(!isTRUE(quiet)){
     # ew're done -- update and close progress bar
@@ -925,6 +990,7 @@ MTLDMA.calc <- function(txt.tokens, factor.size, num.tokens=NULL, min.tokens=9,
     MTLDMA=mtldma.res.value,
     sd=mtldma.res.sd,
     all=all.res,
+    steps=steps,
     lengths=list(
       all=mtldma.len.compl,
       mean=mtldma.len.mean,
@@ -934,3 +1000,28 @@ MTLDMA.calc <- function(txt.tokens, factor.size, num.tokens=NULL, min.tokens=9,
 
   return(mtldma.results)
 } ## end function MTLDMA.calc()
+
+
+## function ttr.calc.chars()
+ttr.calc.chars <- function(tokens, type="TTR", log.base=10, steps, char.steps=5, quiet=FALSE){
+  if(!isTRUE(quiet)){
+    message(paste0(type, ".char: Calculate ",type," values"))
+    # give some feedback, so we know the machine didn't just freeze...
+    prgBar <- txtProgressBar(min=0, max=steps, style=3)
+  } else {}
+  char.results <- t(sapply(1:steps, function(x){
+        curr.token <- x * char.steps
+        if(!isTRUE(quiet)){
+          # update progress bar
+          setTxtProgressBar(prgBar, x)
+        } else {}
+        char.temp <- c(token=curr.token, value=ttr.calc(txt.tokens=tokens[1:curr.token], type=type, log.base=log.base))
+        return(char.temp)
+      }
+    ))
+  if(!isTRUE(quiet)){
+    # close prograss bar
+    close(prgBar)
+  } else {}
+  return(char.results)
+} ## end function ttr.calc.chars()
