@@ -31,36 +31,34 @@ set.lang.support("treetag",
   list("ru"=list(
     ## preset: "ru"
     lang="ru",
-    encoding="",
+    encoding="UTF-8",
     preset=function(TT.cmd, TT.bin, TT.lib, unix.OS){
       if(isTRUE(unix.OS)){
         # preset for unix systems
         return(
           list(
-            TT.tokenizer    = file.path(TT.cmd, "utf8-tokenize.perl"),
-            TT.tagger      = file.path(TT.bin, "tree-tagger"),
-            TT.abbrev      = c(),
-            TT.params      = file.path(TT.lib, "russian.par"),
+            TT.tokenizer      = file.path(TT.cmd, "utf8-tokenize.perl"),
+            TT.tagger         = file.path(TT.bin, "tree-tagger"),
+            TT.abbrev         = c(),
+            TT.params         = file.path(TT.lib, "russian-utf8.par"),
 
-            TT.tknz.opts.def  = c(),
-            TT.tknz.opts    = c(),
-            TT.lookup.command  = c(),
-            TT.filter.command  = c()
+            TT.tknz.opts      = c(),
+            TT.lookup.command = c(),
+            TT.filter.command = c()
           )
         )
       } else {
         # preset for windows systems
         return(
           list(
-            TT.tokenizer    = file.path(TT.cmd, "utf8-tokenize.perl"),
-            TT.tagger      = file.path(TT.bin, "tree-tagger.exe"),
-            TT.abbrev      = c(),
-            TT.params      = file.path(TT.lib, "russian.par"),
+            TT.tokenizer      = file.path(TT.cmd, "utf8-tokenize.perl"),
+            TT.tagger         = file.path(TT.bin, "tree-tagger.exe"),
+            TT.abbrev         = c(),
+            TT.params         = file.path(TT.lib, "russian-utf8.par"),
 
-            TT.tknz.opts.def  = c(),
-            TT.tknz.opts    = c(),
-            TT.lookup.command  = c(),
-            TT.filter.command  = c()
+            TT.tknz.opts      = c(),
+            TT.lookup.command = c(),
+            TT.filter.command = c()
           )
         )
       }
